#include "stdafx.h"
#include "iis4ruby_cmn_err.h"
#include "Message.h"

////////////////////////////////////////////////////////////////
// static function

//
// Name  :	GetUserSID
// Param :	SID& oUserSid
//
//
static void GetUserSID( OUT SID& oUserSid ) {

	HANDLE hCurrentThread ;
	HANDLE hCurrentProcess ;
	HANDLE hToken ; 
	BOOL bResult ;
	DWORD nErr ;
	TOKEN_USER* pTokenUser	= NULL ;
	DWORD dwLength ;

	::ZeroMemory( &oUserSid, sizeof( oUserSid ) ) ;

	hCurrentThread	= ::GetCurrentThread() ;
	bResult	= ::OpenThreadToken( hCurrentThread, TOKEN_QUERY, TRUE, &hToken ) ;
	if( !bResult ) {
		nErr	= ::GetLastError() ;
		if( nErr != ERROR_NO_TOKEN ) {
			return ;
		}

		hCurrentProcess	= ::GetCurrentProcess() ;
		bResult	= ::OpenProcessToken( hCurrentProcess, TOKEN_QUERY, &hToken ) ;
		if( !bResult ) {
			return ;
		}
	}

	::GetTokenInformation( hToken, TokenUser, pTokenUser, 0, &dwLength ) ;
	pTokenUser	= reinterpret_cast<TOKEN_USER*>( new char[ dwLength ] ) ;
	bResult	= ::GetTokenInformation( hToken, TokenUser, pTokenUser, dwLength, &dwLength ) ;
	if( !bResult ) {
		return ;
	}

	::memcpy_s( &oUserSid, sizeof( oUserSid ), pTokenUser->User.Sid, sizeof( oUserSid ) ) ;

	delete[] pTokenUser ;
}

//
// Name  :	RegEventSource
// Param :	HMODULE hModule
//
//
static void RegEventSource( IN HMODULE hModule ) {

	HKEY hRegKey ;
	DWORD dwTypesSupported ;
	DWORD dwCategroyCount ;
	TCHAR wFileName[ 1024 ] ;
	int nLen ;

	::ZeroMemory( wFileName, sizeof( wFileName ) ) ;
	::GetModuleFileName( hModule, wFileName, sizeof( wFileName ) - 1 ) ;
	nLen	= _tcscnlen( wFileName, sizeof( wFileName ) ) * sizeof( TCHAR ) + 1 ;

	// WXgɓo^
	::RegCreateKey( HKEY_LOCAL_MACHINE, REG_EVENTLOG_ROOT EVENT_DATASOURCE, &hRegKey ) ;
	::RegSetValueEx( hRegKey, REG_EVENTLOG_KEY_EVT_MSG_FILE, 0, REG_EXPAND_SZ, (LPBYTE)wFileName, nLen ) ;
	::RegSetValueEx( hRegKey, REG_EVENTLOG_KEY_CAT_MSG_FILE, 0, REG_EXPAND_SZ, (LPBYTE)wFileName, nLen ) ;
	dwTypesSupported	= EVENTLOG_ERROR_TYPE | EVENTLOG_WARNING_TYPE | EVENTLOG_INFORMATION_TYPE | EVENTLOG_SUCCESS ;
	::RegSetValueEx( hRegKey, REG_EVENTLOG_KEY_TYPES_SUPPORTED, 0, REG_DWORD, (LPBYTE)&dwTypesSupported, sizeof( dwTypesSupported ) ) ;
	dwCategroyCount	= CNT_EVENT_CATEGORY ;
	::RegSetValueEx( hRegKey, REG_EVENTLOG_KEY_CAT_COUNT, 0, REG_DWORD, (LPBYTE)&dwCategroyCount, sizeof( dwCategroyCount ) ) ;
	::RegCloseKey( hRegKey ) ;
}

////////////////////////////////////////////////////////////////
// export function

//
// Name  :	ReportEventLog
// Param :	...
//
//
void ReportEventLog( WORD wType, WORD wCategory, DWORD dwEventID,
					const char* pFile, const int iLine, const char* pDate, const char* pTime, LPCTSTR pMsg ) {

	BOOL bResult	= FALSE ;
	HANDLE hEventLog	= NULL ;
	SID oUserSid ;
	PCTSTR lpszEventLogs[ ( sizeof( EVENTLOG_STRINGS ) / sizeof( EVENTLOG_STRINGS[ 0 ] ) ) ] ;
	int i ;
	int iElems	= 0 ;

	::ZeroMemory( lpszEventLogs, sizeof( lpszEventLogs ) ) ;
	iElems	= ( sizeof( EVENTLOG_STRINGS ) / sizeof( EVENTLOG_STRINGS[ 0 ] ) ) ;
	for( i = 0; i < iElems; i++ ) {
		lpszEventLogs[ i ]	= new TCHAR[ MAX_LEN_EVENT_TEXT ] ;
	}

	hEventLog	= ::RegisterEventSource(
		NULL,
		EVENT_DATASOURCE
		) ;
	if( hEventLog == NULL ) {
		return ;
	}

	// SID擾
	GetUserSID( oUserSid ) ;

	// bZ[W쐬
#ifdef UNICODE
	_stprintf_s( (wchar_t*)( lpszEventLogs[ 0 ] ), MAX_LEN_EVENT_TEXT, EVENTLOG_STRINGS[ 0 ], A2T( pFile ), iLine, A2T( pDate ), A2T( pTime ) ) ;
	_stprintf_s( (wchar_t*)( lpszEventLogs[ 1 ] ), MAX_LEN_EVENT_TEXT, EVENTLOG_STRINGS[ 1 ], pMsg ) ;
#else
	wsprintf( lpszEventLogs[ 0 ], EVENTLOG_STRINGS[ 0 ], CA2T( pFile ), iLine, CA2T( pDate ), CA2T( pTime ) ) ;
	wsprintf( lpszEventLogs[ 1 ], EVENTLOG_STRINGS[ 1 ], pMsg ) ;
#endif

	bResult	= ::ReportEvent(
		hEventLog,
		wType,
		wCategory,
		dwEventID,
		&oUserSid,
		2,
		0,
		lpszEventLogs,
		NULL
		) ;
	if( !bResult ) {
		dwEventID	= ::GetLastError() ;
	}

	::DeregisterEventSource( hEventLog ) ;

	for( i = 0; i < iElems; i++ ) {
		delete[] lpszEventLogs[ i ] ;
	}
}

//
// Name  :	ReportEventLogForSystem
// Param :	...
//
//
void ReportEventLogForSystem( const char* pFile, const int iLine, const char* pDate, const char* pTime ) {

	DWORD nErr	= 0L ;
	LPTSTR lpErrMsg ;

	nErr	= ::GetLastError() ;
	::FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM, NULL, nErr, LANG_USER_DEFAULT, 
		(LPTSTR)&lpErrMsg, 0, NULL ) ;

	::ReportEventLog( EVENTLOG_ERROR_TYPE, CAT_ETC, MSG_SYS, pFile, iLine, pDate, pTime, lpErrMsg ) ;

	::LocalFree( lpErrMsg ) ;
}

//
// Name  :	InitCmnLib
// Param :	...
//
//
void InitCmnLib( HMODULE hModule ) {
	RegEventSource( hModule ) ;
}
